

#code SLOW_CODE
;romdate:: dm	__date__,0
;romtime:: dm	__time__,0


#code CODE

#assert BC_IS_VPC	    == 1
#assert SP_IS_VSTACK	    == 1
;#assert IX_IS_RSTACK	    == 1
#assert DE_IS_TOP	    == 1
;#assert HEAP_RETAIN_RELEASE == 1
;#assert HEAP_USES_HANDLES == 1
#assert IY_IS_NEXT == 1


#include "lib-zxsp/screen-macros.s"


; -------------------------------------
; ei ( -- )

ei::	ei
	jp	iy	; next

; -------------------------------------
; di ( -- )

di::	di
	jp	iy	;next

; ________________________________
; wait for next interrupt
; enables interrupts 
;
; wait ( -- )

wait::	ei
	halt
	jp	iy	;next

; -------------------------------------
; in ( u16 -- u8 )

in__6uint16_5uint8::
	ld	hl,bc	; save
	ld	bc,de	; addr
	in	e,(bc)
	ld	bc,hl	; restore
	jp	extend_pos

; _____________________________________
; out ( u16:addr u8:byte -- )

out__6uint16_5uint8_::
	ld	hl,bc	; save
	pop	bc	; bc = Addr
	out	(bc),e
	ld	bc,hl	; restore
	pop	de
	jp	iy	;next

; _____________________________________
; getc  ( -- char )
; waitc ( -- char )
; print ( char -- )
; print ( int -- )
; print ( uint -- )
; print ( char* -- )
; print ( str¢ -- )
; print ( str -- )
	
getc__8ucs1char::
	push	de
	call	get_inkey	; non blocking
	jp	return_a
	
waitc__8ucs1char::
	push	de
	call	wait_inkey	; blocking
	jp	return_a

getc_avail__4bool::
	push	de
	call	get_avail_in
	jp	return_nz
	
print__8ucs1char_::
	ld	a,e
	call	print_char
	pop	de
	jp	iy

print__8ucs1charP_::
	call	print_text_de
	pop	de
	jp	iy

print__6uint16_::
	ex	hl,de		; hl = number
	call	print_uint
	pop	de
	jp	iy
	
print__5int16_::
	ex	hl,de		; hl = number
	call	print_int
	pop	de
	jp	iy
	
print__8ucs1charAE_::
	push	de
	call	pstr
	pop	de
	jp	DROPSTR
	
print__8ucs1charAEC_::
	call	pstr
	pop	de
	jp	iy
	
pstr:	ld	a,d
	or	e
	jp	z, next
	
	deref_handle_de		; de -> text.data

	ld	hl,de
	dec	hl
	dec	hl
	ld	a,(hl)
	jp	print_text_dea


; __________________________________
; print decimal number to buffer
;
; itoa ( hl:number de:buffer -- ) 
; utoa ( hl:number de:buffer -- ) 
 
itoa__5int16_8ucs1charP_::
	pop	hl	; de->buffer, hl=number
	call	itoa
	pop	de
	jp	iy

utoa__6uint16_8ucs1charP_::
	pop	hl	; de->buffer, hl=number
	call	utoa
	pop	de
	jp	iy


; __________________________________
; hide and show mouse
;
; void show_mouse()
; void hide_mouse()

SHOW_MOUSE::
	push	bc
	push	de
	call	show_mouse
	pop	de
	pop	bc
	jp	iy

HIDE_MOUSE::
	push	bc
	push	de
	call	hide_mouse
	pop	de
	pop	bc
	jp_next


locate__6uint16_::
	push	bc
	ld	bc,de
	call	p8_locate
	pop	bc
	pop	de
	jp_next
	
setattr__5uint8_::
	ld	a,e
	call	p8_setattr
	pop	de
	jp_next

clear_background::
	call	print_msg \ db ctl_setattr, white+blue_paper, ctl_cls, 0	
	pop	de
	jp_next
	
; plot(uint x, uint y)
plot__6uint16_6uint16_::
	pop	hl		; l = x
	ld	h,e		; h = y
	push	bc
	call	plot_point_hl
	pop	bc
	pop	de
	jp_next

; draw(uint xy, uint xy)
draw__6uint16_6uint16_::
	pop	hl
	push	bc
	call	draw_line_fast
	pop	bc
	pop	de
	jp_next

draw__6uint16_6uint16_6uint16_6uint16_::
	ld	d,e	; d = y2
	pop	hl
	ld	e,l	; e = x2
	pop	hl
	ld	a,l
	pop	hl	; l = x1
	ld	h,a	; h = y1
	
	push	bc
	call	draw_line_fast
	pop	bc
	pop	de
	jp_next

clear_board__6uint16_::
	push	bc
	ld	bc,de
	call	draw_empty_board
	pop	bc
	pop	de
	jp_next


; set_attr(xy, wh, attr)
; automatic detection of pixel vs. char
; if y > 24 || h > 8 --> pixel hor+ver
; if x > 32 || w > 8 --> pixel hor
set_attr_p__6uint16_6uint16_6uint16_::
	ld	a,e
	pop	de		; de = wh
	pop	hl		; hl = xy
	push	bc		; save PC
	ld	c,a		; C = attr
	call	set_attr_pbox
	pop	bc		; PC
	pop	de		; TOP
	jp_next

set_attr_c__6uint16_6uint16_6uint16_::
	ld	a,e
	pop	de		; de = wh
	pop	hl		; hl = xy
	push	bc		; save PC
	ld	c,a		; C = attr
	call	set_attr_cbox
	pop	bc		; PC
	pop	de		; TOP
	jp_next


; all dimensions are scaled to pixels:
set_attr_pbox:
	ld	a,h
	and	7
	add	d
	add	7
	rra \ rra \ rra
	and	$1f
	ld	d,a		; d = H
	
	ld	a,h
	rra \ rra \ rra
	and	$1f
	ld	h,a		; h = Y

	ld	a,l
	and	7
	add	e
	add	7
	rra \ rra \ rra
	and	$1f
	ld	e,a		; e = W
	
	ld	a,l
	rra \ rra \ rra
	and	$1f
	ld	l,a		; l = X
	;jr set_attr_cbox

; all dimensions are scaled to chars:
set_attr_cbox:
	ld a,d \ and a \ ret z
	ld a,e \ and a \ ret z	; security

	calc_attr_addr_for_row_col H,L

; paint with attr C:
	
	ld	a,l		; a = attr_addr.lo
6$:	ld	b,e		; b = cols
5$:	ld	(hl),c 
	inc 	l
	djnz	5$
	dec	d		; next row
	ret	z		; end
	add	32
	ld	l,a		; attr_addr += 32
	jr	nc,6$
	inc	h
	jr	6$


; UP:
txt_calc_hl_from_row_b_col_c::
	calc_scr_addr_for_row_col B,C
	ret


; void draw_digit(uint xy, uint digit)
draw_digit__6uint16_6uint16_::
	pop	hl				; xy
	push	bc
	ld	bc,hl
	call	txt_calc_hl_from_row_b_col_c	; hl -> screen byte
	call	draw_char_de_at_hl
	pop	bc
	pop	de
	jp_next

; draw 16x16 char A at position row B, col C
; draw 16x16 char A at screen byte position HL
;
; DE = 0      = free
; DE = 1 .. 9 = '1' to '9'
; bit 4: preset digit
; bits 7 … 15 = pencil

draw_char_de_at_hl:
	ld	a,e		; char
	and	$1f		; note: preserve bit 4
	jr	z, no_char
		
	; calc glyph address:
	add a \ add a
	add a \ add a 		; a = char*16; cy = bit 4
	ld	d,0
	ld	e,a
	ex	hl,de		; de -> screen, hl = char*16
	ld	bc,zif0
	jr	c, bold_char
	;jr	nc,normal_char

; draw a normal char for a user entered digit:
normal_char:
	add	hl,hl		; *32
	add	hl,bc		; hl -> glyph
	call	1$
	call	adjust_de_for_next_row
1$:	ld	b,8
2$:	ld	a,(hl) \ inc l
	ld	(de),a \ inc e
	ld	a,(hl) \ inc hl
	ld	(de),a \ dec e
	inc	d			; next pixel row inside char8 row
	djnz	2$
	ret

; draw a bold char for a preset position:
bold_char:
	add	hl,hl		; *32
	add	hl,bc		; hl -> glyph
	call	1$
	call	adjust_de_for_next_row
1$:	and	a
	ld	b,8	

2$:	ld	a,(hl) \ rra \ or (hl)
	ld 	(de),a \ inc e
	ld	a,(hl) \ rra \ inc l		; 'or(hl)' mach das cy kaputt
	ld	a,(hl) \ rra \ or (hl) \ inc hl
	ld	(de),a \ dec e
	
	inc	d			; next pixel row inside char8 row
	djnz	2$
	ret

; draw an empty cell or some pencil bits:
no_char:
	ld	a,e
	and	$80			; bits 5 and 6 may be used for m_selected and m_error: mask them off
	or	d
	jr	nz,draw_pencil

; draw an empty cell:
clear_cell:
	call	1$
	call	adjust_hl_for_next_row
1$:	xor	a
	ld	b,8
2$:	ld	(hl),a \ inc l
	ld	(hl),a \ dec l
	inc	h			; next pixel row inside char8 row
	djnz	2$
	ret

; draw pencil bits de[15…7]:
draw_pencil:
	push	hl
	call	clear_cell
	pop	hl

	inc	h
	push	hl	; + 1 row

	ld	a,h
	add	5
	ld	h,a	; + 6 rows
	push	hl

	add	5
	ld	h,a	; + 11 rows
	call	adjust_hl_for_next_row

	ld	bc, zif0 + 11*2		; 7 8 9 -> start in row[11]
	sla	e
	rl	d
	call	4$

	pop	hl
	ld	bc, zif0 + 6*2		; 4 5 6 -> start in row[6]
	call	1$

	pop	hl
	ld	bc, zif0 + 1*2		; 1 2 3 -> start in row[1]
	;jr	1$

; draw right bytes of pencil glyph:
; d = pencil bits, shifted left
; bc -> pencil glyph
; hl -> screen byte

1$:	sla	d		; test for highest = right digit

4$:	push	hl		; hl -> screen
	push	bc		; bc -> glyph
	inc	c
	inc	l

	jr	nc, 6$		; no right digit
	bit 	7,d
	jr	z, 5$		; right no middle

; right and 1/2 middle digit:
	call	draw_unmasked_pencil
	jr	2$

; right no middle:
5$:	ld	e, $1E
	call	draw_masked_pencil
	jr	2$

; no right digit:
6$:	bit	7,d
	ld	e, $C0
	call	nz, draw_masked_pencil	; middle no right
	;jr	2$

2$:	pop	bc
	pop	hl

; draw left bytes of pencil glyph:

	sla	d
	jr	nc, 3$		; no middle digit

; middle digit:
	sla	d		; left digit?
	jr	c, draw_unmasked_pencil	; left and middle digit
	ld	e, $03
	jr	draw_masked_pencil	; middle no left digit
	
; no middle digit:
3$:	sla	d
	ret	nc		; no left no middle digit
	
; left no middle digit:
	ld	e,$78		; mask
	;jr	draw_masked_pencil

draw_masked_pencil:
	; e = mask
	; hl -> screen
	; bc -> glyph
	; pres: d
	
	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	and 	e
	ld	(hl),a
	inc	h

	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	and 	e
	ld	(hl),a
	inc	h

	; when drawing pencil 4 5 6 then we moved here into the next char row:
	ld	a,h
	and	7
	call	z,adjust_hl_for_next_row
	
	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	and 	e
	ld	(hl),a
	inc	h

	ld	a,(bc)		; bc -> glyph
	and 	e
	ld	(hl),a
	ret

draw_unmasked_pencil:
	; hl -> screen
	; bc -> glyph
	; pres: d
	
	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	ld	(hl),a
	inc	h

	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	ld	(hl),a
	inc	h

	; when drawing pencil 4 5 6 then we moved here into the next char row:
	ld	a,h
	and	7
	call	z,adjust_hl_for_next_row
	
	ld	a,(bc)		; bc -> glyph
	inc bc \ inc bc
	ld	(hl),a
	inc	h

	ld	a,(bc)		; bc -> glyph
	ld	(hl),a
	ret

	

	
	
; mul9 ( uint -- uint )
mul9::	ld	hl,de
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,de
	ex	hl,de
	jp_next


	


	


	
	
	
	

